unit SpEdit;
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls , Speech, IniFiles, Routine;
type
  TSpEdit = class(TEdit)
  private
    FOnEnter    : TNotifyEvent;
    FOnKeyDown  : TKeyEvent;
    FOnKeyUp    : TKeyEvent;
    FOnKeyPress    : TKeyPressEvent;
    FSpeechEnabled : boolean;
    FSpeechName    : string;
    FIniFileName   : string;

    FSpeech : TSpeech;
    procedure SetSpeech(const Value: TSpeech);
  protected
    function  GetSpeechName : string ;
    procedure SetSpeechName( Name : string );

    function  GetIniFileName : string ;
    procedure SetIniFileName( Name : string );

  public
    SelMode  : boolean;
    SelBegin : word;
    FSpeakMode     : boolean;

    constructor Create(AOwner: TComponent); override;
    procedure DoEnter; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure SetSpeechEnabled( Flag : boolean );
    procedure ConfigureComponent;
  published

    property OnEnter   : TNotifyEvent read FOnEnter   write FOnEnter;
    property OnKeyUp   : TKeyEvent read FOnKeyUp   write FOnKeyUp;
    property OnKeyDown : TKeyEvent read FOnKeyDown write FOnKeyDown;
    property OnKeyPress : TKeyPressEvent read FOnKeyPress write FOnKeyPress;
    property Speech:TSpeech read FSpeech write SetSpeech;

    property SpeechEnabled : boolean read FSpeechEnabled write SetSpeechEnabled default False;
    property SpeechName: string read GetSpeechName write SetSpeechName;
    property IniFileName: string read GetIniFileName write SetIniFileName;
    property SpeakMode:boolean read FSpeakMode write FSpeakMode;
  end;

implementation
var
   SaveCaretPos : word;
constructor TSpEdit.Create(AOwner: TComponent);
begin
 inherited Create( AOwner );
 ConfigureComponent;

end;

procedure TSpEdit.DoEnter;
begin
 Speech.Stop;
 if length(Text) > 20 then  Speech.Speak(FSpeechName + '  ')
    else
     if Text='' then  Speech.Speak(FSpeechName + ' ')
                else  Speech.Speak(FSpeechName + ' ' + Text );
end;

procedure TSpEdit.KeyUp(var Key: Word; Shift: TShiftState);
var
 CutString  : string;
 StrPointer : integer;
begin
 inherited;
 Case Key of
  VK_HOME, VK_END, VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN : Speech.Stop;
 end;


 if Key = VK_HOME then Speech.Speak(' ');
 if Key = VK_END  then Speech.Speak(' ');

 if ( Key = VK_LEFT ) or ( Key = VK_RIGHT ) or
    ( Key = VK_UP ) or ( key = VK_DOWN ) then

  if ssCtrl in Shift then
    begin
      if Text = '' then Speech.Speak(' ')
       else
        begin
         StrPointer := SaveCaretPos;
         CutString := '';

         if Key = VK_RIGHT then inc(StrPointer);
         if Key = VK_LEFT  then dec(StrPointer);

         repeat
          dec(StrPointer);
         until ( StrPointer < 1 ) or (Text[StrPointer] = #32 );

         inc(StrPointer);
         repeat
          CutString := CutString + Text[StrPointer];
          inc(StrPointer);
         until ( StrPointer > length( Text )) or (Text[StrPointer]= #32);

         Speech.Speak(CutString);
       end;
     end
   else
   if Text = '' then Speech.Speak(' ')
                      else
                       begin
                        if SelStart> 0 then Speech.Speak( Text[SelStart] );
                        if (( Key = VK_RIGHT ) or ( Key = VK_DOWN )) and ( SelStart=Length(Text)) then Speech.Speak(' ');
                        if (( Key = VK_LEFT ) or ( Key = VK_UP )) and ( SelStart=0 ) then Speech.Speak(' ');
                       end;
  if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;

procedure TSpEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
 inherited;
  SaveCaretPos := SelStart;
  if key = VK_BACK then
    if Text = '' then Speech.Speak(' ')
                 else Speech.Speak(Text[SelStart]);

  if key = VK_DELETE then
    if Text = '' then Speech.Speak(' ')
         else
          if SelStart = length(Text) then Speech.Speak(' ')
                                     else Speech.Speak(Text[SelStart+1]);

 if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
end;

procedure TSpEdit.KeyPress( var Key: Char);
begin
 inherited;
 if FSpeakMode then
  begin
   if Key <> chr(VK_BACK) then Speech.Stop;
   Speech.Speak(Key);
  end;

 if Assigned(FOnKeyPress) then FOnKeyPress(Self, Key);
end;

procedure TSpEdit.SetSpeechEnabled( Flag : boolean );
begin
 FSpeechEnabled := Flag ;
end;

function  TSpEdit.GetSpeechName : string ;
begin
 GetSpeechName := FSpeechName;
end;

procedure TSpEdit.SetSpeechName( Name : string );
begin
 FSpeechName := Name;
end;

function  TSpEdit.GetIniFileName : string ;
begin
 GetIniFileName := FIniFileName;
end;

procedure TSpEdit.SetIniFileName( Name : string );
begin
 FIniFileName := Name ;
 ConfigureComponent;
end;



procedure TSpEdit.SetSpeech(const Value: TSpeech);
begin
  FSpeech := Value;
  Refresh
end;

procedure TSpEdit.ConfigureComponent;
begin
 if Parent = nil then Exit;
// Text := Routine.ConfigureComponent( FIniFileName, Parent.GetNamePath, GetNamePath, Text );
 SpeechName := Routine.ConfigureComponent( FIniFileName, Parent.GetNamePath, GetNamePath+'_sn', FSpeechName );
end;

end.