unit speech;
interface
uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   ComObj, ActiveX, SAPI;
type
  SingleRulesRec = record
    c    : char;
    name : string;
  end;

  MultiRulesRec = record
    word, name : string;
  end;


   TSpeechEngineInfo = record
      Name:             String;
      Language:         String;
      Manufacturer:     String;
      Product:          String;
      ModeID:           String;
      EngineID:         String;
      Speaker:          String;
      Style:            String;
      Gender:           String;
      Age:              Integer;
      Features:         Integer;
      Interfaces:       Integer;
      MaxPitch:         Word;
      MinPitch:         Word;
      MaxRealTime:      Longword;
      MinRealTime:      Longword;
      MaxSpeed:         Longword;
      MinSpeed:         Longword;
      MaxVolume:        Longword;
      MinVolume:        Longword;
   end;

   TSpeechDialog = (sdAbout, sdGeneral, sdLexicon, sdTranslate);
   TSpeechOutput = (soFile, soGeneral);

   TMSAChangedEvent = procedure(Sender: TObject; Attribute: Longword) of object;
   TMSErrorEvent = procedure(Sender: TObject; Code: Integer; Message: String) of object;
   TMSVisualEvent = procedure(Sender: TObject; Hints: Longword; Mouth: PTTSMouth) of object;

type
   ESpeechException = class(Exception);
   TSpeech = class;


   TTSNotifySink = class(TInterfacedObject, ITTSNotifySink2)
   private
      FOwner:                  TSpeech;
   protected
      function AttribChanged(dwAttribute: DWORD): HResult; stdcall;
      function AudioStart(qTimeStamp: QWORD): HResult; stdcall;
      function AudioStop(qTimeStamp: QWORD): HResult; stdcall;
      function Visual(qTimeStamp: QWORD;
                      cIPAPhoneme: AnsiChar;
                      cEnginePhoneme: AnsiChar;
                      dwHints: DWORD;
                      apTTSMouth: PTTSMouth): HResult; stdcall;

      function Error(Error: IUnknown): HResult; stdcall;
      function Warning(Warning: IUnknown): HResult; stdcall;
      function VisualFuture(dwMilliseconds: DWORD;
                            qTimeStamp: QWORD;
                            cIPAPhoneme: AnsiChar;
                            cEnginePhoneme: AnsiChar;
                            dwHints: DWORD;
                            pTTSMouth: PTTSMouth): HResult; stdcall;
   public
      constructor Create(AOwner: TSpeech);
   end;

   TTSBufNotifySink = class(TInterfacedObject, ITTSBufNotifySink)
   private
      FOwner:                  TSpeech;
   protected
      function TextDataDone(qTimeStamp: QWORD; dwFlags: DWORD): HResult; stdcall;
      function TextDataStarted(qTimeStamp: QWORD): HResult; stdcall;
      function BookMark(qTimeStamp: QWORD; dwMarkNum: DWORD): HResult; stdcall;
      function WordPosition(qTimeStamp: QWORD; dwByteOffset: DWORD): HResult; stdcall;
   public
      constructor Create(AOwner: TSpeech);
   end;

   TSpeech = class(TComponent)
   private
      FEnabled:                Boolean;
      FEngines:                TStrings;
      FEngineStarted:          Boolean;
      FFile:                   WideString;
      FIAF:                    IAudioFile;
      FIAMD:                   IAudioMultimediaDevice;
      FInfo:                   TSpeechEngineInfo;
      FInit:                   Boolean;
      FITTSAttributes:         ITTSAttributesA;
      FITTSCentral:            ITTSCentralA;
      FITTSDialogs:            ITTSDialogs;
      FITTSEnum:               ITTSEnum;
      FKey:                    Longword;
      FOutput:                 TSpeechOutput;
      FPaused:                 Boolean;
      FText:                   TStrings;
      FTTSNotifySink:          ITTSNotifySink2;
      FTTSBufNotifySink:       ITTSBufNotifySink;
      FVersion, DummyS:        String;
      PModeInfo:               PTTSModeInfo;

      function GetInterface(Index: Integer): IUnknown;
      function GetPitch: Word;
      function GetRealTime: Longword;
      function GetSpeed: Longword;
      function GetVolume: Longword;
      procedure SetText(const Value: TStrings);
      procedure SetPitch(const Value: Word);
      procedure SetRealTime(const Value: Longword);
      procedure SetSpeed(const Value: Longword);
      procedure SetVolume(const Value: Longword);

      procedure BeforeSelectEngine;
      procedure Init;
      procedure InitAudio;
      procedure PostSelectEngine;
      procedure PostSelectEngine2(ModeInfo: TTSModeInfo);

      function GetAudioDevice: IUnknown;
   protected
      FOnAttribChanged:        TMSAChangedEvent;
      FOnError:                TMSErrorEvent;
      FOnStart:                TNotifyEvent;
      FOnStop:                 TNotifyEvent;
      FOnWarning:              TMSErrorEvent;
      FOnVisual:               TMSVisualEvent;

      FPos:                    Longword;

      procedure FlushFile;
   public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Inject(Command: String);
      procedure Pause;
      procedure Resume;
      procedure SelectEngine(EngineName: String); overload;
      procedure SelectEngine(EngineMode: TTSModeInfo); overload;
      procedure ShowDialog(DialogType: TSpeechDialog; ParentWnd: HWND = 0;
                           Title: PChar = nil);
      procedure Speak; overload;
      procedure Speak(const strText: String); overload;
      procedure Stop;

      function GetEngineInfo(EngineName: String; var Info: TSpeechEngineInfo): Boolean;

      property Interfaces[Index: Integer]: IUnknown read GetInterface;
      property Paused: Boolean read FPaused;
      property Pitch: Word read GetPitch write SetPitch;
      property Position: Longword read FPos;
      property Engines: TStrings read FEngines;
      property Info: TSpeechEngineInfo read FInfo;
      property RealTime: Longword read GetRealTime write SetRealTime;
      property Speed: Longword read GetSpeed write SetSpeed;
      property Volume: Longword read GetVolume write SetVolume;
///

    function  SingleFilter(Text : string ):string;
    function  MultiFilter(Text : string ):string;
    procedure InitRules;
    procedure SaveSingleRules( FileName : string );
    procedure LoadSingleRules( FileName : string );
    procedure SaveMultiRules( FileName : string );
    procedure LoadMultiRules( FileName : string );

   published
      property Enabled: Boolean read FEnabled write FEnabled;
      property Filename: WideString read FFile write FFile;
      property Output: TSpeechOutput read FOutput write FOutput;
      property Text: TStrings read FText write SetText;
      property Version: String read FVersion write DummyS;
      property OnAttribChanged: TMSAChangedEvent read FOnAttribChanged write FOnAttribChanged;
      property OnError: TMSErrorEvent read FOnError write FOnError;
      property OnStart: TNotifyEvent read FOnStart write FOnStart;
      property OnStop: TNotifyEvent read FOnStop write FOnStop;
      property OnWarning: TMSErrorEvent read FOnWarning write FOnWarning;
      property OnVisual: TMSVisualEvent read FOnVisual write FOnVisual;
   end;
var

  SingleRules : array [0..255] of SingleRulesRec;
  SingleRulesCount : byte;

  MultiRules : array [0..$FFFF] of MultiRulesRec;
  MultiRulesCount : word;


implementation
{$R *.res}
constructor TSpeech.Create(AOwner: TComponent);
begin
   inherited;
   FEngines := TStringList.Create;
   FText := TStringList.Create;
   FKey := 0;
   FPos := 0;
   FOutput := soGeneral;
   FEnabled := True;
   FPaused := False;
   FVersion := 'TSpeech 1.0';
   Init;
end;

destructor TSpeech.Destroy;
begin
   FInit := False;
   FEngines.Free;
   FText.Free;

   if Assigned(PModeInfo) then Dispose(PModeInfo);

   inherited;
end;

procedure TSpeech.Init;
var ModeInfo: TTSModeInfo;
    EngineCount: Integer;
begin
   FInit := True;
   FEngineStarted := False;

   try
      { Enumerate engines }
      OleCheck(CoCreateInstance(CLSID_TTSEnumerator, Nil, CLSCTX_ALL, IID_ITTSEnum, FITTSEnum));
      OleCheck(FITTSEnum.Reset);
      OleCheck(FITTSEnum.Next(1, ModeInfo, @EngineCount));

      while EngineCount > 0 do
      begin
         FEngines.Add(String(ModeInfo.szModeName));
         OleCheck(FITTSEnum.Next(1, ModeInfo, @EngineCount));
      end;
   except
      FInit := False;
      FEnabled := False;
   end;
end;

procedure TSpeech.InitAudio;
begin
   case Output of
      soFile:
         OleCheck(CoCreateInstance(CLSID_AudioDestFile, nil, CLSCTX_ALL,
                  IID_IAudioFile, FIAF));
      soGeneral:
         OleCheck(CoCreateInstance(CLSID_MMAudioDest, nil, CLSCTX_ALL,
                  IID_IAudioMultiMediaDevice, FIAMD));
   end;
end;

function TSpeech.GetAudioDevice;
begin
   case Output of
      soFile:
         Result := FIAF;
      soGeneral:
         Result := FIAMD;
   end;
end;

procedure TSpeech.FlushFile;
begin
   if FOutput = soFile then FIAF.Flush; // close file
end;

procedure TSpeech.Inject;
begin
   FITTSCentral.Inject(PChar(Command));
end;

procedure TSpeech.BeforeSelectEngine;
begin
   { Check if audio device is available }
   InitAudio;

   { Unregister old notify interface }
   if FKey > 0 then
   begin
      FITTSCentral.UnRegister(FKey);
      FKey := 0;
   end;

   { Create notify interfaces }
   FTTSBufNotifySink := TTSBufNotifySink.Create(Self);
   FTTSNotifySink := TTSNotifySink.Create(Self);
end;

procedure TSpeech.SelectEngine(EngineName: String);
var Index, EngineCount: Integer;
    ModeInfo: TTSModeInfo;
begin
   if not FInit then
      raise ESpeechException.Create('msgSENotInited');

   Index := Engines.IndexOf(EngineName);
   if Index < 0 then
      raise ESpeechException.Create('msgSENotFound');

   { Select Engine }
   FEngineStarted := True;
   try
      BeforeSelectEngine;
      OleCheck(FITTSEnum.Reset);
      OleCheck(FITTSEnum.Skip(Index));
      OleCheck(FITTSEnum.Next(1, ModeInfo, @EngineCount));

      if Assigned(PModeInfo) then Dispose(PModeInfo);
      New(PModeInfo);
      PModeInfo^ := ModeInfo;

      OleCheck(FITTSEnum.Select(PModeInfo^.gModeID,
               FITTSCentral, GetAudioDevice));

      PostSelectEngine;
      PostSelectEngine2(ModeInfo);
   except
      FEngineStarted := False;
   end;
end;

procedure TSpeech.SelectEngine(EngineMode: TTSModeInfo);
var FITTSFind: ITTSFind;
    ModeInfo: TTSModeInfo;
begin
   if not FInit then
      raise ESpeechException.Create('msgSENotInited');

   { Find and Select Engine }
   FEngineStarted := True;
   try
      try
         BeforeSelectEngine;
         OleCheck(CoCreateInstance(CLSID_TTSEnumerator, nil, CLSCTX_ALL,
                  IID_ITTSFind, FITTSFind));
         OleCheck(FITTSFind.QueryInterface(IID_ITTSEnum, FITTSEnum));
         OleCheck(FITTSFind.Find(EngineMode, nil, ModeInfo));

         if Assigned(PModeInfo) then Dispose(PModeInfo);
         New(PModeInfo);
         PModeInfo^ := ModeInfo;

         OleCheck(FITTSFind.Select(PModeInfo^.gModeID,
                  FITTSCentral, GetAudioDevice));

         PostSelectEngine;
         PostSelectEngine2(ModeInfo);
      except
         FEngineStarted := False;
      end;
   finally
      FITTSFind._Release;
   end;
end;

procedure TSpeech.PostSelectEngine;
begin
   OleCheck(FITTSCentral.QueryInterface(IID_ITTSAttributes, FITTSAttributes));
   OleCheck(FITTSCentral.QueryInterface(IID_ITTSDialogs, FITTSDialogs));

   if FITTSCentral.Register(Pointer(FTTSNotifySink),
                            IID_ITTSNotifySink2, FKey) <> 0 then
   begin
      OleCheck(FITTSCentral.Register(Pointer(FTTSNotifySink),
                                     IID_ITTSNotifySink, FKey));
   end;
end;

procedure TSpeech.PostSelectEngine2;
var CurrentPitch: Word;
    CurrentRealTime, CurrentSpeed: Longword;
begin
   { Retrieve Engine Information }
   FInfo.Language := StrPas(ModeInfo.Language.szDialect);
   with FInfo, ModeInfo do
   begin
      Name := StrPas(szModeName);
      Manufacturer := StrPas(szMfgName);
      Product := StrPas(szProductName);
      ModeID := GUIDToString(gModeID);
      EngineID := GUIDToString(gEngineID);
      Speaker := StrPas(szSpeaker);
      Style := StrPas(szStyle);

      case wGender of
         0: Gender := 'NEUTRAL';
         1: Gender := 'FEMALE';
         2: Gender := 'MALE';
      end;

      Age := wAge;
      Features := dwFeatures;
      Interfaces := dwInterfaces;

      with FITTSAttributes do
      begin
         PitchGet(CurrentPitch);
         PitchSet(TTSATTR_MAXPITCH);
         PitchGet(MaxPitch);
         PitchSet(TTSATTR_MINPITCH);
         PitchGet(MinPitch);
         PitchSet(CurrentPitch);

         RealTimeGet(CurrentRealTime);
         RealTimeSet(TTSATTR_MAXREALTIME);
         RealTimeGet(MaxRealTime);
         RealTimeSet(TTSATTR_MINREALTIME);
         RealTimeGet(MinRealTime);
         RealTimeSet(CurrentRealTime);

         SpeedGet(CurrentSpeed);
         SpeedSet(TTSATTR_MAXSPEED);
         SpeedGet(MaxSpeed);
         SpeedSet(TTSATTR_MINSPEED);
         SpeedGet(MinSpeed);
         SpeedSet(CurrentSpeed);

         { According to MS's help file }
         MaxVolume := 100;
         MinVolume := 0;
      end;
   end;
end;

procedure TSpeech.Pause;
begin
   OleCheck(FITTSCentral.AudioPause);
   FPaused := True;
end;

procedure TSpeech.Resume;
begin
   OleCheck(FITTSCentral.AudioResume);
   FPaused := False;
end;

procedure TSpeech.ShowDialog;
begin
   case DialogType of
      sdAbout: OleCheck(FITTSDialogs.AboutDlg(ParentWnd, Title));
      sdGeneral: OleCheck(FITTSDialogs.GeneralDlg(ParentWnd, Title));
      sdLexicon: OleCheck(FITTSDialogs.LexiconDlg(ParentWnd, Title));
      sdTranslate: OleCheck(FITTSDialogs.TranslateDlg(ParentWnd, Title));
   end;
end;

procedure TSpeech.Speak;
begin
   Speak(FText.Text);
end;

procedure TSpeech.Speak(const strText: String);
var
    SData : TSData;
    s : string;
begin
 if strText = '' then Exit;
 if length(strText)=1 then s := SingleFilter(strText)
  else
   begin
    s := MultiFilter(strText);
   end;

   if not FEnabled then Exit;

   if not FInit then
      raise ESpeechException.Create('msgSENotInited');

   if not FEngineStarted then
      raise ESpeechException.Create('msgSENotStarted');

   FPaused := False;
// OleCheck(FITTSCentral.AudioReset);

   try
      if FOutput = soFile then
         if FIAF.DoSet(PWideChar(Filename), 1) < 0 then
            raise ESpeechException.Create('msgCannotOpenFile');

      SData.dwSize := Length(s) + 1;
      SData.pData := PChar(s);
      OleCheck(FITTSCentral.TextData (CHARSET_TEXT, 0,
               SData, Pointer(FTTSBufNotifySink), IID_ITTSBufNotifySink));
   except
   end;
end;

procedure TSpeech.Stop;
begin
   FPaused := False;
   OleCheck(FITTSCentral.AudioReset);
end;

function TSpeech.GetEngineInfo;
var Index, EngineCount: Integer;
    ModeInfo: TTSModeInfo;
begin
   Result := False;

   if not FInit then Exit;
   Index := Engines.IndexOf(EngineName);
   if Index < 0 then Exit;

   if FITTSEnum.Reset < 0 then Exit;
   if FITTSEnum.Skip(Index) < 0 then Exit;
   if FITTSEnum.Next(1, ModeInfo, @EngineCount) < 0 then Exit;

   { Retrieve Engine Information }
   FillChar(Result, Sizeof(Result), 0);
   Info.Language := StrPas(ModeInfo.Language.szDialect);
   with Info, ModeInfo do
   begin
      Name := StrPas(szModeName);
      Manufacturer := StrPas(szMfgName);
      Product := StrPas(szProductName);
      ModeID := GUIDToString(gModeID);
      EngineID := GUIDToString(gEngineID);
      Speaker := StrPas(szSpeaker);
      Style := StrPas(szStyle);

      case wGender of
         0: Gender := 'NEUTRAL';
         1: Gender := 'FEMALE';
         2: Gender := 'MALE';
      end;

      Age := wAge;
      Features := dwFeatures;
      Interfaces := dwInterfaces;

      MaxVolume := 100;
   end;
   Result := True;
end;

{ Setting/Getting properties }

{ Undocumented property, for advanced user only }
function TSpeech.GetInterface;
begin
   case Index of
      0: Result := GetAudioDevice;
      1: Result := FITTSCentral;
      2: Result := FITTSAttributes;
      3: Result := FITTSDialogs;
   end;
end;

function TSpeech.GetPitch;
begin
   FITTSAttributes.PitchGet(Result);
end;

procedure TSpeech.SetPitch;
begin
   FITTSCentral.Inject(PChar('\Pit=' + IntToStr(Value) + '\'));
// FITTSAttributes.PitchSet(Value);
end;

function TSpeech.GetRealTime;
begin
   FITTSAttributes.RealTimeGet(Result);
end;

procedure TSpeech.SetRealTime;
begin
   FITTSAttributes.RealTimeSet(Value);
end;

function TSpeech.GetSpeed;
begin
   FITTSAttributes.SpeedGet(Result);
end;


procedure TSpeech.SetSpeed;
begin
   FITTSCentral.Inject(PChar('\Spd=' + IntToStr(Value) + '\'));
// FITTSAttributes.SpeedSet(Value);
end;

{$ifdef MANUAL_SET_AND_GET_VOLUME}
function TSpeech.GetVolume;
begin
   FITTSAttributes.VolumeGet(Result);
end;

procedure TSpeech.SetVolume;
begin
   FITTSCentral.Inject(PChar('\Vol=' + IntToStr(Volume) + '\'));
end;
{$else}
function TSpeech.GetVolume;
var CurrentVolume: Longword;
begin
   FITTSAttributes.VolumeGet(CurrentVolume);
   Result := LoWord(CurrentVolume) * 100 div 65535;
end;

procedure TSpeech.SetVolume;
var TempVolume: Longword;
begin
   TempVolume := 65535 * Value div 100;
   TempVolume := MakeWParam(TempVolume, TempVolume);
   FITTSCentral.Inject(PChar('\Vol=' + IntToStr(TempVolume) + '\'));
// OleCheck(FITTSAttributes.VolumeSet(TempVolume));
end;
{$endif}

procedure TSpeech.SetText;
begin
   FText.Assign(Value);
end;

{ TTSNotifySink }

constructor TTSNotifySink.Create;
begin
   FOwner := AOwner;
end;

function TTSNotifySink.AttribChanged;
begin
   if Assigned(FOwner.FOnAttribChanged) then FOwner.FOnAttribChanged(Self, dwAttribute);
   Result := 0;
end;

function TTSNotifySink.AudioStart;
begin
   if Assigned(FOwner.FOnStart) then FOwner.FOnStart(Self);
   Result := 0;
end;

function TTSNotifySink.AudioStop;
begin
   FOwner.FlushFile;
   if Assigned(FOwner.FOnStop) then FOwner.FOnStop(Self);
   Result := 0;
end;

function TTSNotifySink.Visual;
begin
   if Assigned(FOwner.FOnVisual) then FOwner.FOnVisual(Self, dwHints, apTTSMouth);
   Result := 0;
end;

{ TTSNotifySink2 }

function TTSNotifySink.Error;
var Err: SpchError;
begin
   if Assigned(FOwner.FOnError) then
   begin
      ISpchError(Error).LastErrorGet(Err);
      FOwner.FOnError(Self, Err.hRes, Err.szStrings);
   end;
   Result := 0;
end;

function TTSNotifySink.VisualFuture;
begin
   Result := 0;
end;

function TTSNotifySink.Warning;
var Warn: SpchError;
begin
   if Assigned(FOwner.FOnWarning) then
   begin
      ISpchError(Warning).LastErrorGet(Warn);
      FOwner.FOnWarning(Self, Warn.hRes, Warn.szStrings);
   end;
   Result := 0;
end;

{ TTSBufNotifySink }

constructor TTSBufNotifySink.Create;
begin
   FOwner := AOwner;
end;

function TTSBufNotifySink.BookMark;
begin
   Result := 0;
end;

function TTSBufNotifySink.TextDataDone;
begin
   Result := 0;
end;

function TTSBufNotifySink.TextDataStarted;
begin
   Result := 0;
end;

function TTSBufNotifySink.WordPosition;
begin
   FOwner.FPos := dwByteOffset;
   Result := 0;
end;

/////////////////////////////////////////////////////////////////
{procedure TSpeech.FilterSpeak( text : string );
var
 s : string;
begin
 if text = '' then Exit;
 if length(text)=1 then s := SingleFilter(Text)
  else
   begin
    s := MultiFilter(Text);
   end;

 if s <> '' then Speak(s);
end;}

function TSpeech.SingleFilter(Text : string ):string;
var
 i : integer;
 s : string;
begin
 if text = '' then Exit;
 if length(text)>1 then Exit;
 // Added leter
 if Text[1] = #$0D then Text := '';

 s := '';
 for i:=0 to SingleRulesCount-1 do
  if SingleRules[i].c = text then s := SingleRules[i].name;


 if s='' then SingleFilter := Text
         else SingleFilter := s;

end;

function TSpeech.MultiFilter(Text : string ):string;
var
 l,j,k,e,i : integer;
 s1,s2,s3 : string;
begin
 if text = '' then Exit;
 if length(text)=1 then Exit;
 s1 := text;
 for i:=0 to MultiRulesCount-1 do
  begin
   l := length( MultiRules[i].word );
   k := 0;
   j :=1;
   repeat
    s2 := '';
    for e := j to j+l-1 do s2 := s2 + s1[e];
    if AnsiUpperCase(s2) = AnsiUpperCase(MultiRules[i].word) then k := j;

    if k > 0 then
     begin
      s2 := '';
      s3 := '';
      for e:=1 to k-1 do s2 := s2 + s1[e];
      for e:=k+l to length(s1) do s3 := s3 + s1[e];
      s1 := s2 + MultiRules[i].name + s3;
      k:=0;
      j:=j+length(MultiRules[i].name);
     end;

     inc(j);
    until j > length(s1)-l+1;
    end;


 MultiFilter := s1;

end;


procedure TSpeech.InitRules;
begin
//
end;




procedure TSpeech.SaveSingleRules( FileName : string );
var
 F : TextFile;
 i : integer;
begin
 AssignFile(F,FileName);
 Rewrite(F);
 for i:=0 to SingleRulesCount-1 do
  begin
   write(F,#39+SingleRules[i].c + #39+'=' + SingleRules[i].name + #$0D + #$0A);
  end;
 CloseFile(F);
end;

procedure TSpeech.LoadSingleRules( FileName : string );
var
 F : TextFile;
 i : integer;
 s : string;
begin
 AssignFile(F,FileName);
 Reset(F);
 SingleRulesCount := 0;
{$I-}
 repeat
  readln(F,s);
  if  s <> ''  then
   begin
    inc(SingleRulesCount);
    SingleRules[SingleRulesCount-1].c := s[2];
    SingleRules[SingleRulesCount-1].name := '';
    for i:=5 to length(s) do SingleRules[SingleRulesCount-1].name := SingleRules[SingleRulesCount-1].name + s[i];
   end;
 until  s='';

 CloseFile(F);
end;

procedure TSpeech.SaveMultiRules( FileName : string );
var
 F : TextFile;
 i : integer;
begin
 AssignFile(F,FileName);
 Rewrite(F);
 for i:=0 to MultiRulesCount-1 do
  begin
   write(F,MultiRules[i].word + '=' + MultiRules[i].name + #$0D + #$0A);
  end;
 CloseFile(F);
end;

procedure TSpeech.LoadMultiRules( FileName : string );
var
 F : TextFile;
 i : integer;
 s : string;
begin
 AssignFile(F,FileName);
 Reset(F);
 MultiRulesCount := 0;
{$I-}
 repeat
  readln(F,s);
  if  s <> ''  then
   begin
    inc(MultiRulesCount);
    i := 1;
    MultiRules[MultiRulesCount-1].word := '';
    MultiRules[MultiRulesCount-1].name := '';
    repeat
     if s[i]<>'=' then
         MultiRules[MultiRulesCount-1].word := MultiRules[MultiRulesCount-1].word + s[i];
     inc(i);
    until s[i]='=';
    inc(i);
    repeat
     if s[i]<>'=' then
         MultiRules[MultiRulesCount-1].name := MultiRules[MultiRulesCount-1].name + s[i];
     inc(i);
    until i > length(s);
   end;
 until  s='';
 CloseFile(F);
end;



end.
