unit SpListBox;
interface
uses
  Windows,SysUtils, Classes, Controls, StdCtrls,Speech;

const
  cSpeechName = 'List box';

type
  TSpListBox = class(TListBox)
  private
    FOnEnter    : TNotifyEvent;
    FOnKeyDown  : TKeyEvent;
    FOnKeyUp    : TKeyEvent;
    FOnKeyPress : TKeyPressEvent;

    FSpeechEnabled : boolean;
    FSpeechName    : string ;
    NameFlag       : boolean ;
    FSpeech : TSpeech;
    procedure SetSpeech(const Value: TSpeech);
  protected
    function  GetSpeechName : string ;
    procedure SetSpeechName( Name : string );
  public
    SelMode  : boolean;
    SelBegin : word;

    constructor Create(AOwner: TComponent); override;
    procedure DoEnter; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState);override;
    procedure KeyDown(var Key: Word; Shift: TShiftState);override;
    procedure KeyPress(var Key: Char); override;
    procedure SetSpeechEnabled( Flag : boolean );
  published

    property OnEnter   : TNotifyEvent read FOnEnter   write FOnEnter;
    property OnKeyUp   : TKeyEvent read FOnKeyUp   write FOnKeyUp;
    property OnKeyDown : TKeyEvent read FOnKeyDown write FOnKeyDown;
    property Speech:TSpeech read FSpeech write SetSpeech;

    property SpeechEnabled : boolean read FSpeechEnabled write SetSpeechEnabled default False;
    property SpeechName: string read GetSpeechName write SetSpeechName ;
  end;

implementation
var
   SaveCaretPos : word;
constructor TSpListBox.Create(AOwner: TComponent);
begin
 inherited Create( AOwner );
end;

procedure TSpListBox.DoEnter;
begin
 Speech.Stop;
 if ItemIndex > -1 then Speech.Speak(FSpeechName + Items[ItemIndex])
                  else Speech.Speak(FSpeechName);
end;

procedure TSpListBox.KeyUp(var Key: Word; Shift: TShiftState);
var
 CutString  : string;
 StrPointer : integer;
begin
 inherited;
 Case Key of
  VK_HOME, VK_END, VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN : Speech.Stop;
 end;

 if ( Key = VK_LEFT ) or ( Key = VK_RIGHT ) or
    ( Key = VK_UP ) or ( key = VK_DOWN ) then
     if ItemIndex > -1 then Speech.Speak(Items[ItemIndex]);

 if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;

procedure TSpListBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
 inherited;
 if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
//
end;

procedure TSpListBox.KeyPress( var Key: Char);
begin
 inherited;
//
end;

procedure TSpListBox.SetSpeechEnabled( Flag : boolean );
begin
 FSpeechEnabled := Flag ;
end;

function  TSpListBox.GetSpeechName : string ;
begin
 if NameFlag = false then
  begin
   FSpeechName := cSpeechName;
   NameFlag := true ;
  end;
 GetSpeechName := FSpeechName;
end;

procedure TSpListBox.SetSpeechName( Name : string );
begin
 FSpeechName := Name ;
end;


procedure TSpListBox.SetSpeech(const Value: TSpeech);
begin
  FSpeech := Value;
  Refresh
end;


end.
