unit SpListView;
interface
uses
  SysUtils, Classes, Controls, ComCtrls,Speech;
const
 ListViewSpeechName = 'List View';

type
  TSpListView = class(TListView)
  private
    FOnEnter    : TNotifyEvent;
    FOnKeyUp    : TKeyEvent;
    FOnClick    : TNotifyEvent;
    FSpeech : TSpeech;
    FSpeechEnabled : boolean;
    FSpeechName    : string ;
    NameFlag       : boolean ;
    procedure SetSpeech(const Value: TSpeech);
    procedure Speak( Text : string );
  protected
    function  GetSpeechName : string ;
    procedure SetSpeechName( Name : string );
  public
    constructor Create(AOwner: TComponent); override;
    procedure DoEnter; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState);override;
    procedure Click; override;
  published
    property OnEnter : TNotifyEvent read FOnEnter   write FOnEnter;
    property OnKeyUp : TKeyEvent read FOnkeyUp   write FOnKeyUp;
    property OnClick : TNotifyEvent read FOnClick   write FOnClick;

    property Speech:TSpeech read FSpeech write SetSpeech;

    property SpeechEnabled : boolean read FSpeechEnabled write FSpeechEnabled default False;
    property SpeechName: string read GetSpeechName write SetSpeechName ;

  end;



implementation

constructor TSpListView.Create(AOwner: TComponent);
begin
 inherited Create( AOwner );
end;

procedure TSpListView.DoEnter;
begin
 inherited;
 Speech.Stop;
 if Selected<>nil then Speak(FSpeechName + ' ' + Selected.Caption )
                  else Speak(FSpeechName);
end;

procedure TSpListView.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Selected<>nil then Speak(FSpeechName + ' ' + Selected.Caption )
                   else Speak(FSpeechName);
  if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;


procedure TSpListView.Click;
begin
 inherited;
 if Selected<>nil then Speak( Selected.Caption )
end;


procedure TSpListView.SetSpeech(const Value: TSpeech);
begin
  FSpeech := Value;
  Refresh;
end;

procedure TSpListView.Speak( Text : string );
begin
 if Speech = nil then Exit;
 Speech.Speak(Text);
end;


function  TSpListView.GetSpeechName : string ;
begin
 if NameFlag = false then
  begin
   FSpeechName := ListViewSpeechName;
   NameFlag := true ;
  end;
 GetSpeechName := FSpeechName;
end;

procedure TSpListView.SetSpeechName( Name : string );
begin
 FSpeechName := Name ;
end;


end.
