unit SpRichEdit;
interface
uses
  Windows,SysUtils, Classes, Controls, StdCtrls, ComCtrls,Speech;
const
  cSpeechName = 'rich edit';
type
  TSpRichEdit = class(TRichEdit)
    private
    FOnEnter    : TNotifyEvent;
    FOnKeyDown  : TKeyEvent;
    FOnKeyUp    : TKeyEvent;
    FOnKeyPress : TKeyPressEvent;

    FSpeechEnabled : boolean;
    FSpeechName    : string ;
    NameFlag       : boolean ;
    FSpeech : TSpeech;
    procedure SetSpeech(const Value: TSpeech);
  protected
    function  GetSpeechName : string ;
    procedure SetSpeechName( Name : string );
  public
    SelMode  : boolean;
    SelBegin : word;

    constructor Create(AOwner: TComponent); override;
    procedure DoEnter; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState);override;
    procedure KeyDown(var Key: Word; Shift: TShiftState);override;
    procedure KeyPress(var Key: Char); override;
    procedure SetSpeechEnabled( Flag : boolean );
  published

    property OnEnter   : TNotifyEvent read FOnEnter   write FOnEnter;
    property OnKeyUp   : TKeyEvent read FOnKeyUp   write FOnKeyUp;
    property OnKeyDown : TKeyEvent read FOnKeyDown write FOnKeyDown;
    property Speech:TSpeech read FSpeech write SetSpeech;

    property SpeechEnabled : boolean read FSpeechEnabled write SetSpeechEnabled default False;
    property SpeechName: string read GetSpeechName write SetSpeechName ;
  end;

implementation
var
   SaveCaretPos : word;
constructor TSpRichEdit.Create(AOwner: TComponent);
begin
 inherited Create( AOwner );
end;

procedure TSpRichEdit.DoEnter;
begin
 inherited;
 Speech.Stop;
 if length(Text) > 0 then  Speech.Speak(FSpeechName +  Lines[CaretPos.y] )
                     else Speech.Speak(FSpeechName )
end;

procedure TSpRichEdit.KeyUp(var Key: Word; Shift: TShiftState);
var
 CutString  : string;
 StrPointer : integer;
begin
 inherited;
 Case Key of
  VK_HOME, VK_END, VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN : Speech.Stop;
 end;


 if Key = VK_HOME then Speech.Speak(' ');
 if Key = VK_END  then Speech.Speak(' ');

 if ( Key = VK_UP ) or ( key = VK_DOWN ) then
  begin
   Speech.Speak(Lines[CaretPos.y]);
  end;

 if ( Key = VK_LEFT ) or ( Key = VK_RIGHT ) then


  if ssCtrl in Shift then
    begin
      if Text = '' then Speech.Speak(' ')
       else
        begin
         StrPointer := SaveCaretPos;
         CutString := '';

         if Key = VK_RIGHT then inc(StrPointer);
         if Key = VK_LEFT  then dec(StrPointer);

         repeat
          dec(StrPointer);
         until ( StrPointer < 1 ) or (Text[StrPointer] = #32 );

         inc(StrPointer);
         repeat
          CutString := CutString + Text[StrPointer];
          inc(StrPointer);
         until ( StrPointer > length( Text )) or (Text[StrPointer]= #32);

         Speech.Speak(CutString);
       end;
     end
   else
   if Text = '' then Speech.Speak(' ')
                      else
                       begin
                        if SelStart> 0 then Speech.Speak( Text[SelStart] );
                        if (( Key = VK_RIGHT ) or ( Key = VK_DOWN )) and ( SelStart=Length(Text)) then Speech.Speak(' ');
                        if (( Key = VK_LEFT ) or ( Key = VK_UP )) and ( SelStart=0 ) then Speech.Speak(' ');
                       end;
  if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;

procedure TSpRichEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
 inherited;
  SaveCaretPos := SelStart;
  if key = VK_BACK then
    if Text = '' then Speech.Speak(' ')
                 else Speech.Speak(Text[SelStart]);

  if key = VK_DELETE then
    if Text = '' then Speech.Speak(' ')
         else
          if SelStart = length(Text) then Speech.Speak(' ')
                                     else Speech.Speak(Text[SelStart+1]);

  if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
end;

procedure TSpRichEdit.KeyPress( var Key: Char);
var
 Filter : string;
begin
 inherited;
 if Key <> chr(VK_BACK) then Speech.Stop;
 case Key of
  #32 : Filter := '';
  '' : Filter := '';
  '' : Filter := '';
  '' : Filter := '';
  '' : Filter := '';
  '$' : Filter := '';
  '%' : Filter := '';
 else
  filter := key;
 end;
 Speech.Speak(Filter);

  if Assigned(FOnKeyPress) then FOnKeyPress(Self, Key ); 
end;

procedure TSpRichEdit.SetSpeechEnabled( Flag : boolean );
begin
 FSpeechEnabled := Flag ;
end;

function  TSpRichEdit.GetSpeechName : string ;
begin
 if NameFlag = false then
  begin
   FSpeechName := cSpeechName;
   NameFlag := true ;
  end;
 GetSpeechName := FSpeechName;
end;

procedure TSpRichEdit.SetSpeechName( Name : string );
begin
 FSpeechName := Name ;
end;


procedure TSpRichEdit.SetSpeech(const Value: TSpeech);
begin
  FSpeech := Value;
  Refresh
end;


end.
