unit SpTreeView;
interface
uses
  SysUtils, Classes, Controls, ComCtrls,Speech;
const
 TreeViewSpeechName = 'Tree View';

type
  TSpTreeView = class(TTreeView)
  private
    FOnEnter    : TNotifyEvent;
    FOnKeyUp    : TKeyEvent;
    FOnClick    : TNotifyEvent;
    FSpeech : TSpeech;
    FSpeechEnabled : boolean;
    FSpeechName    : string ;
    NameFlag       : boolean ;
    procedure SetSpeech(const Value: TSpeech);
    procedure Speak( Text : string );
  protected
    function  GetSpeechName : string ;
    procedure SetSpeechName( Name : string );
  public
    constructor Create(AOwner: TComponent); override;
    procedure DoEnter; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState);override;
    procedure Click; override;
  published
    property OnEnter : TNotifyEvent read FOnEnter   write FOnEnter;
    property OnKeyUp : TKeyEvent read FOnkeyUp   write FOnKeyUp;    
    property OnClick : TNotifyEvent read FOnClick   write FOnClick;

    property Speech:TSpeech read FSpeech write SetSpeech;

    property SpeechEnabled : boolean read FSpeechEnabled write FSpeechEnabled default False;
    property SpeechName: string read GetSpeechName write SetSpeechName ;

  end;



implementation

constructor TSpTreeView.Create(AOwner: TComponent);
begin
 inherited Create( AOwner );
end;

procedure TSpTreeView.DoEnter;
begin
  Speech.Stop;
  if Selected<>nil then Speak(FSpeechName + ' ' + Selected.Text )
                   else Speak(FSpeechName);
end;

procedure TSpTreeView.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if Selected<>nil then Speak( Selected.Text )
                   else Speak( FSpeechName + ' ' );

 if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;


procedure TSpTreeView.Click;
begin
 inherited;
 if Selected<>nil then
  begin
   Speak( Selected.Text );
  end;
 if Assigned(FOnClick) then FOnClick(Self);
end;


procedure TSpTreeView.SetSpeech(const Value: TSpeech);
begin
  FSpeech := Value;
  Refresh;
end;

procedure TSpTreeView.Speak( Text : string );
begin
 if Speech = nil then Exit;
 Speech.Stop;
 Speech.Speak(Text);
end;


function  TSpTreeView.GetSpeechName : string ;
begin
 if NameFlag = false then
  begin
   FSpeechName := TreeViewSpeechName;
   NameFlag := true ;
  end;
 GetSpeechName := FSpeechName;
end;

procedure TSpTreeView.SetSpeechName( Name : string );
begin
 FSpeechName := Name ;
end;



end.
